@@ -32,64 +32,64 @@
 #endif
 
 #include "mbfilter.h"
-#include "mbfilter_iso8859_16.h"
-#include "unicode_table_iso8859_16.h"
+#include "mbfilter_iso8859_10.h"
+#include "unicode_table_iso8859_10.h"
 
-static const char *mbfl_encoding_8859_16_aliases[] = {"ISO_8859-16", NULL};
+static const char *mbfl_encoding_8859_10_aliases[] = {"ISO_8859-10", "latin6", NULL};
 
-const mbfl_encoding mbfl_encoding_8859_16 = {
-	mbfl_no_encoding_8859_16,
-	"ISO-8859-16",
-	"ISO-8859-16",
-	(const char *(*)[])&mbfl_encoding_8859_16_aliases,
+const mbfl_encoding mbfl_encoding_8859_10 = {
+	mbfl_no_encoding_8859_10,
+	"ISO-8859-10",
+	"ISO-8859-10",
+	(const char *(*)[])&mbfl_encoding_8859_10_aliases,
 	NULL,
 	MBFL_ENCTYPE_SBCS,
-	&vtbl_8859_16_wchar,
-	&vtbl_wchar_8859_16
+	&vtbl_8859_10_wchar,
+	&vtbl_wchar_8859_10
 };
 
-const struct mbfl_identify_vtbl vtbl_identify_8859_16 = {
-	mbfl_no_encoding_8859_16,
+const struct mbfl_identify_vtbl vtbl_identify_8859_10 = {
+	mbfl_no_encoding_8859_10,
 	mbfl_filt_ident_common_ctor,
 	mbfl_filt_ident_common_dtor,
 	mbfl_filt_ident_true
 };
 
-const struct mbfl_convert_vtbl vtbl_8859_16_wchar = {
-	mbfl_no_encoding_8859_16,
+const struct mbfl_convert_vtbl vtbl_8859_10_wchar = {
+	mbfl_no_encoding_8859_10,
 	mbfl_no_encoding_wchar,
 	mbfl_filt_conv_common_ctor,
 	mbfl_filt_conv_common_dtor,
-	mbfl_filt_conv_8859_16_wchar,
+	mbfl_filt_conv_8859_10_wchar,
 	mbfl_filt_conv_common_flush
 };
 
-const struct mbfl_convert_vtbl vtbl_wchar_8859_16 = {
+const struct mbfl_convert_vtbl vtbl_wchar_8859_10 = {
 	mbfl_no_encoding_wchar,
-	mbfl_no_encoding_8859_16,
+	mbfl_no_encoding_8859_10,
 	mbfl_filt_conv_common_ctor,
 	mbfl_filt_conv_common_dtor,
-	mbfl_filt_conv_wchar_8859_16,
+	mbfl_filt_conv_wchar_8859_10,
 	mbfl_filt_conv_common_flush
 };
 
 #define CK(statement)	do { if ((statement) < 0) return (-1); } while (0)
 
 /*
- * ISO-8859-16 => wchar
+ * ISO-8859-10 => wchar
  */
-int mbfl_filt_conv_8859_16_wchar(int c, mbfl_convert_filter *filter)
+int mbfl_filt_conv_8859_10_wchar(int c, mbfl_convert_filter *filter)
 {
 	int s;
 
 	if (c >= 0 && c < 0xa0) {
 		s = c;
 	} else if (c >= 0xa0 && c < 0x100) {
-		s = iso8859_16_ucs_table[c - 0xa0];
+		s = iso8859_10_ucs_table[c - 0xa0];
 		if (s <= 0) {
 			s = c;
 			s &= MBFL_WCSPLANE_MASK;
-			s |= MBFL_WCSPLANE_8859_16;
+			s |= MBFL_WCSPLANE_8859_10;
 		}
 	} else {
 		s = c;
@@ -103,9 +103,9 @@ int mbfl_filt_conv_8859_16_wchar(int c, mbfl_convert_filter *filter)
 }
 
 /*
- * wchar => ISO-8859-16
+ * wchar => ISO-8859-10
  */
-int mbfl_filt_conv_wchar_8859_16(int c, mbfl_convert_filter *filter)
+int mbfl_filt_conv_wchar_8859_10(int c, mbfl_convert_filter *filter)
 {
 	int s, n;
 
@@ -115,13 +115,13 @@ int mbfl_filt_conv_wchar_8859_16(int c, mbfl_convert_filter *filter)
 		s = -1;
 		n = 95;
 		while (n >= 0) {
-			if (c == iso8859_16_ucs_table[n]) {
+			if (c == iso8859_10_ucs_table[n]) {
 				s = 0xa0 + n;
 				break;
 			}
 			n--;
 		}
-		if (s <= 0 && (c & ~MBFL_WCSPLANE_MASK) == MBFL_WCSPLANE_8859_16) {
+		if (s <= 0 && (c & ~MBFL_WCSPLANE_MASK) == MBFL_WCSPLANE_8859_10) {
 			s = c & MBFL_WCSPLANE_MASK;
 		}
 	}
