@@ -236,11 +236,12 @@ static inline void accel_restart_enter(void)
 #ifdef ZEND_WIN32
 	INCREMENT(restart_in);
 #else
-# ifdef _AIX
-	static FLOCK_STRUCTURE(restart_in_progress, F_WRLCK, SEEK_SET, 2, 1);
-# else
-	static const FLOCK_STRUCTURE(restart_in_progress, F_WRLCK, SEEK_SET, 2, 1);
-#endif
+	struct flock restart_in_progress;
+
+	restart_in_progress.l_type = F_WRLCK;
+	restart_in_progress.l_whence = SEEK_SET;
+	restart_in_progress.l_start = 2;
+	restart_in_progress.l_len = 1;
 
 	if (fcntl(lock_file, F_SETLK, &restart_in_progress) == -1) {
 		zend_accel_error(ACCEL_LOG_DEBUG, "RestartC(+1):  %s (%d)", strerror(errno), errno);
@@ -255,11 +256,12 @@ static inline void accel_restart_leave(void)
 	ZCSG(restart_in_progress) = 0;
 	DECREMENT(restart_in);
 #else
-# ifdef _AIX
-	static FLOCK_STRUCTURE(restart_finished, F_UNLCK, SEEK_SET, 2, 1);
-# else
-	static const FLOCK_STRUCTURE(restart_finished, F_UNLCK, SEEK_SET, 2, 1);
-# endif
+	struct flock restart_finished;
+
+	restart_finished.l_type = F_UNLCK;
+	restart_finished.l_whence = SEEK_SET;
+	restart_finished.l_start = 2;
+	restart_finished.l_len = 1;
 
 	ZCSG(restart_in_progress) = 0;
 	if (fcntl(lock_file, F_SETLK, &restart_finished) == -1) {
@@ -272,7 +274,12 @@ static inline int accel_restart_is_active(void)
 {
 	if (ZCSG(restart_in_progress)) {
 #ifndef ZEND_WIN32
-		FLOCK_STRUCTURE(restart_check, F_WRLCK, SEEK_SET, 2, 1);
+		struct flock restart_check;
+
+		restart_check.l_type = F_WRLCK;
+		restart_check.l_whence = SEEK_SET;
+		restart_check.l_start = 2;
+		restart_check.l_len = 1;
 
 		if (fcntl(lock_file, F_GETLK, &restart_check) == -1) {
 			zend_accel_error(ACCEL_LOG_DEBUG, "RestartC:  %s (%d)", strerror(errno), errno);
@@ -297,11 +304,12 @@ static inline int accel_activate_add(void)
 #ifdef ZEND_WIN32
 	INCREMENT(mem_usage);
 #else
-# ifdef _AIX
-	static FLOCK_STRUCTURE(mem_usage_lock, F_RDLCK, SEEK_SET, 1, 1);
-# else
-	static const FLOCK_STRUCTURE(mem_usage_lock, F_RDLCK, SEEK_SET, 1, 1);
-# endif
+	struct flock mem_usage_lock;
+
+	mem_usage_lock.l_type = F_RDLCK;
+	mem_usage_lock.l_whence = SEEK_SET;
+	mem_usage_lock.l_start = 1;
+	mem_usage_lock.l_len = 1;
 
 	if (fcntl(lock_file, F_SETLK, &mem_usage_lock) == -1) {
 		zend_accel_error(ACCEL_LOG_DEBUG, "UpdateC(+1):  %s (%d)", strerror(errno), errno);
@@ -320,11 +328,12 @@ static inline void accel_deactivate_sub(void)
 		ZCG(counted) = 0;
 	}
 #else
-# ifdef _AIX
-	static FLOCK_STRUCTURE(mem_usage_unlock, F_UNLCK, SEEK_SET, 1, 1);
-# else
-	static const FLOCK_STRUCTURE(mem_usage_unlock, F_UNLCK, SEEK_SET, 1, 1);
-# endif
+	struct flock mem_usage_unlock;
+
+	mem_usage_unlock.l_type = F_UNLCK;
+	mem_usage_unlock.l_whence = SEEK_SET;
+	mem_usage_unlock.l_start = 1;
+	mem_usage_unlock.l_len = 1;
 
 	if (fcntl(lock_file, F_SETLK, &mem_usage_unlock) == -1) {
 		zend_accel_error(ACCEL_LOG_DEBUG, "UpdateC(-1):  %s (%d)", strerror(errno), errno);
@@ -337,11 +346,12 @@ static inline void accel_unlock_all(void)
 #ifdef ZEND_WIN32
 	accel_deactivate_sub();
 #else
-# ifdef _AIX
-	static FLOCK_STRUCTURE(mem_usage_unlock_all, F_UNLCK, SEEK_SET, 0, 0);
-# else
-	static const FLOCK_STRUCTURE(mem_usage_unlock_all, F_UNLCK, SEEK_SET, 0, 0);
-# endif
+	struct flock mem_usage_unlock_all;
+
+	mem_usage_unlock_all.l_type = F_UNLCK;
+	mem_usage_unlock_all.l_whence = SEEK_SET;
+	mem_usage_unlock_all.l_start = 0;
+	mem_usage_unlock_all.l_len = 0;
 
 	if (fcntl(lock_file, F_SETLK, &mem_usage_unlock_all) == -1) {
 		zend_accel_error(ACCEL_LOG_DEBUG, "UnlockAll:  %s (%d)", strerror(errno), errno);
@@ -421,7 +431,7 @@ static zend_always_inline zend_string *accel_find_interned_string(zend_string *s
 	}
 
 	if (!ZCG(counted)) {
-		if (accel_activate_add() == FAILURE) {
+		if (!ZCG(accelerator_enabled) || accel_activate_add() == FAILURE) {
 			return str;
 		}
 		ZCG(counted) = 1;
@@ -562,10 +572,7 @@ static void accel_copy_permanent_strings(zend_new_interned_string_func_t new_int
 	/* empty string */
 	zend_empty_string = new_interned_string(zend_empty_string);
 	for (j = 0; j < 256; j++) {
-		char s[2];
-		s[0] = j;
-		s[1] = 0;
-		zend_one_char_string[j] = new_interned_string(zend_string_init(s, 1, 0));
+		zend_one_char_string[j] = new_interned_string(ZSTR_CHAR(j));
 	}
 	for (j = 0; j < ZEND_STR_LAST_KNOWN; j++) {
 		zend_known_strings[j] = new_interned_string(zend_known_strings[j]);
@@ -729,12 +736,11 @@ static zend_string* ZEND_FASTCALL accel_replace_string_by_shm_permanent(zend_str
 static zend_string* ZEND_FASTCALL accel_replace_string_by_process_permanent(zend_string *str)
 {
 	zend_string *ret = zend_interned_string_find_permanent(str);
-
 	if (ret) {
 		zend_string_release(str);
 		return ret;
 	}
-	ZEND_ASSERT(0);
+	ZEND_ASSERT(!IS_ACCEL_INTERNED(str));
 	return str;
 }
 
@@ -830,8 +836,12 @@ static inline int accel_is_inactive(void)
 		return SUCCESS;
 	}
 #else
-	FLOCK_STRUCTURE(mem_usage_check, F_WRLCK, SEEK_SET, 1, 1);
+	struct flock mem_usage_check;
 
+	mem_usage_check.l_type = F_WRLCK;
+	mem_usage_check.l_whence = SEEK_SET;
+	mem_usage_check.l_start = 1;
+	mem_usage_check.l_len = 1;
 	mem_usage_check.l_pid = -1;
 	if (fcntl(lock_file, F_GETLK, &mem_usage_check) == -1) {
 		zend_accel_error(ACCEL_LOG_DEBUG, "UpdateC:  %s (%d)", strerror(errno), errno);
@@ -1154,7 +1164,7 @@ char *accel_make_persistent_key(const char *path, size_t path_length, int *key_l
 			cwd_len = ZSTR_LEN(cwd_str);
 			if (ZCG(cwd_check)) {
 				ZCG(cwd_check) = 0;
-				if ((ZCG(counted) || ZCSG(accelerator_enabled))) {
+				if (ZCG(accelerator_enabled)) {
 
 					zend_string *str = accel_find_interned_string(cwd_str);
 					if (!str) {
@@ -1194,7 +1204,7 @@ char *accel_make_persistent_key(const char *path, size_t path_length, int *key_l
 
 			if (ZCG(include_path_check)) {
 				ZCG(include_path_check) = 0;
-				if ((ZCG(counted) || ZCSG(accelerator_enabled))) {
+				if (ZCG(accelerator_enabled)) {
 
 					zend_string *str = accel_find_interned_string(ZCG(include_path));
 					if (!str) {
@@ -1277,7 +1287,7 @@ int zend_accel_invalidate(const char *filename, size_t filename_len, zend_bool f
 	zend_string *realpath;
 	zend_persistent_script *persistent_script;
 
-	if (!ZCG(enabled) || !accel_startup_ok || !ZCSG(accelerator_enabled) || accelerator_shm_read_lock() != SUCCESS) {
+	if (!ZCG(accelerator_enabled) || accelerator_shm_read_lock() != SUCCESS) {
 		return FAILURE;
 	}
 
@@ -1900,7 +1910,7 @@ zend_op_array *persistent_compile_file(zend_file_handle *file_handle, int type)
 	} else if (file_cache_only) {
 		return file_cache_compile_file(file_handle, type);
 #endif
-	} else if ((!ZCG(counted) && !ZCSG(accelerator_enabled)) ||
+	} else if (!ZCG(accelerator_enabled) ||
 	           (ZCSG(restart_in_progress) && accel_restart_is_active())) {
 #ifdef HAVE_OPCACHE_FILE_CACHE
 		if (ZCG(accel_directives).file_cache) {
@@ -2195,12 +2205,11 @@ static int persistent_stream_open_function(const char *filename, zend_file_handl
 /* zend_resolve_path() replacement for PHP 5.3 and above */
 static zend_string* persistent_zend_resolve_path(const char *filename, size_t filename_len)
 {
-	if (ZCG(enabled) && accel_startup_ok &&
+	if (
 #ifdef HAVE_OPCACHE_FILE_CACHE
 		!file_cache_only &&
 #endif
-	    (ZCG(counted) || ZCSG(accelerator_enabled)) &&
-	    !ZCSG(restart_in_progress)) {
+	    ZCG(accelerator_enabled)) {
 
 		/* check if callback is called from include_once or it's a main request */
 		if ((!EG(current_execute_data) &&
@@ -2349,6 +2358,7 @@ static void accel_activate(void)
 					zend_alter_ini_entry_chars(key, "0", 1, ZEND_INI_SYSTEM, ZEND_INI_STAGE_RUNTIME);
 					zend_string_release_ex(key, 0);
 					zend_accel_error(ACCEL_LOG_WARNING, "Can't cache files in chroot() directory with too big inode");
+					ZCG(accelerator_enabled) = 0;
 					return;
 				}
 			}
@@ -2406,12 +2416,15 @@ static void accel_activate(void)
 				}
 				accel_restart_leave();
 			}
-		} else {
+		}
+		if (!ZCG(pcre_reseted)) {
 			reset_pcre = 1;
 		}
 		zend_shared_alloc_unlock();
 	}
 
+	ZCG(accelerator_enabled) = ZCSG(accelerator_enabled);
+
 	SHM_PROTECT();
 	HANDLE_UNBLOCK_INTERRUPTIONS();
 
@@ -2423,8 +2436,10 @@ static void accel_activate(void)
 		realpath_cache_clean();
 
 		accel_reset_pcre_cache();
+		ZCG(pcre_reseted) = 0;
 	} else if (reset_pcre) {
 		accel_reset_pcre_cache();
+		ZCG(pcre_reseted) = 1;
 	}
 }
 
@@ -2452,10 +2467,6 @@ static void accel_deactivate(void)
 		zend_string_release_ex(ZCG(cwd), 0);
 		ZCG(cwd) = NULL;
 	}
-
-	if (!ZCG(enabled) || !accel_startup_ok) {
-		return;
-	}
 }
 
 static int accelerator_remove_cb(zend_extension *element1, zend_extension *element2)
